"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Listr = require('listr');
require("rxjs/add/operator/do");
require("rxjs/add/operator/toPromise");
var Observable_1 = require("rxjs/Observable");
var config_file_1 = require("../config-file");
var hg_1 = require("../hg");
// TODO refactor this observable goop.
function merge(config) {
    var projTasks = config.projects.map(function (project) {
        return {
            title: project.path,
            task: function (_, projectTask) {
                return new Listr([
                    {
                        title: 'Pulling...',
                        task: function () { return hg_1.pullChanges(project.path); }
                    },
                    {
                        title: 'Merging...',
                        task: function (ctx, task) { return tryMergeProjectTask(project, ctx, task, projectTask); }
                    }
                ], { concurrent: false, exitOnError: true });
            }
        };
    });
    var tasks = new Listr(projTasks, {
        concurrent: config.maxConcurrent || hg_1.MAX_CONCURRENT_HGS,
        exitOnError: false
    });
    return tasks.run();
}
exports.merge = merge;
function tryMergeProjectTask(project, ctx, task, projectTask) {
    return Observable_1.Observable.create(function (subscriber) {
        checkIfNeedsMerge(project, ctx, task, projectTask)
            .subscribe(function (data) { return subscriber.next(data); }, function (err) { return subscriber.error(err); }, function () {
            if (ctx.isSkipped) {
                delete ctx.isSkipped;
                subscriber.complete();
            }
            else {
                var projectTitleWithBranch_1 = projectTask.title;
                projectTask.title = projectTitleWithBranch_1 + " (merging in " + project.defaultBranch + ")";
                hg_1.mergeBranchIn(project.path, project.defaultBranch)
                    .subscribe(function (data) { return subscriber.next(data); }, function (err) { return subscriber.error(err); }, function () {
                    hg_1.commit(project.path, "Merged in " + project.defaultBranch)
                        .subscribe(function (data) { return subscriber.next(data); }, function (err) { return subscriber.error(err); }, function () {
                        projectTask.title = projectTitleWithBranch_1 + " (merged & committed)";
                        subscriber.complete();
                    });
                });
            }
        });
    });
}
function checkIfNeedsMerge(project, ctx, _, projectTask) {
    return Observable_1.Observable.create(function (subscriber) {
        var branchName;
        hg_1.getBranch(project.path)
            .subscribe(function (data) {
            branchName = data;
            subscriber.next(branchName);
        }, function (err) { return subscriber.error(err); }, function () {
            branchName = (branchName || '').trim();
            projectTask.title = project.path + " - " + branchName;
            if (isExcludedBranch(project, branchName)) {
                ctx.isSkipped = true;
                ctx.isExcludedBranch = true;
                projectTask.skip("...because current branch is " + branchName);
                subscriber.complete();
            }
            else {
                var status_1;
                hg_1.getStatus(project.path)
                    .subscribe(function (data) {
                    status_1 = data;
                    subscriber.next(status_1);
                }, function (err) { return subscriber.error(err); }, function () {
                    status_1 = (status_1 || '').trim();
                    if (status_1.length) {
                        ctx.isSkipped = true;
                        projectTask.skip("...because there are uncommitted changes");
                        projectTask.title = project.path + " - " + branchName + " (" + status_1.length + ")";
                        subscriber.complete();
                    }
                    else {
                        var branchNeedsMerging_1;
                        subscriber.next("Checking if branch already has \"" + project.defaultBranch + "\" merged in...");
                        hg_1.checkIfBranchNeedsOtherBranchMergingIn(project.path, branchName, project.defaultBranch)
                            .subscribe(function (data) { return branchNeedsMerging_1 = data; }, function (err) { return subscriber.error(err); }, function () {
                            if (!branchNeedsMerging_1) {
                                ctx.isSkipped = true;
                                projectTask.skip("...because latest \"" + project.defaultBranch + "\" is already merged in");
                            }
                            subscriber.complete();
                        });
                    }
                });
            }
        });
    });
    function isExcludedBranch(project, branchName) {
        return project.defaultBranch === branchName
            || branchName === config_file_1.DEFAULT_BRANCH_NAME; // todo second check maybes not necessary?
    }
}
