"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var os = require("os");
var CONFIG_FILE_NAME = ".hgpp";
exports.DEFAULT_BRANCH_NAME = 'default';
function getConfig() {
    return new Promise(function (resolve, reject) {
        var fullConfigPath = path.resolve(os.homedir(), CONFIG_FILE_NAME);
        fs.exists(fullConfigPath, function (exists) {
            if (!exists) {
                // TODO make default config file
                reject(new Error(".hgpp config file not found at " + fullConfigPath));
                return;
            }
            fs.readFile(fullConfigPath, function (err, buffer) {
                if (err) {
                    reject(new Error("Error reading .hgpp config file from " + fullConfigPath + ". Error: " + err));
                    return;
                }
                var config;
                try {
                    var configFileText = buffer.toString();
                    config = JSON.parse(configFileText);
                }
                catch (exception) {
                    reject(new Error("Error parsing .hgpp config file. Error: " + err));
                    return;
                }
                config.projects = config.projects.map(function (proj) {
                    proj.defaultBranch = proj.defaultBranch || exports.DEFAULT_BRANCH_NAME;
                    return proj;
                });
                resolve(config);
            });
        });
    });
}
exports.getConfig = getConfig;
