"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var child_process = require("child_process");
var Observable_1 = require("rxjs/Observable");
exports.MAX_CONCURRENT_HGS = 3;
function getBranch(projPath) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg branch...');
        hgCommandToObservable(subscriber, ['branch'], spawnOpts);
    });
}
exports.getBranch = getBranch;
function getStatus(projPath) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        hgCommandToObservable(subscriber, ['status'], spawnOpts);
    });
}
exports.getStatus = getStatus;
function pullChanges(projPath) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg pull...');
        hgCommandToObservable(subscriber, ['pull'], spawnOpts);
    });
}
exports.pullChanges = pullChanges;
function pushChanges(projPath) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next('Running hg push...');
        hgCommandToObservable(subscriber, ['push'], spawnOpts);
    });
}
exports.pushChanges = pushChanges;
function checkIfBranchNeedsOtherBranchMergingIn(projPath, branchName, defaultBranch) {
    return Observable_1.Observable.create(function (subscriber) {
        var hasFoundRevisions = false;
        Observable_1.Observable.create(function (innerSubscriber) {
            var spawnOpts = { cwd: path.resolve(projPath) };
            var revQuery = "not ancestors(heads(" + branchName + ")) and head() and not closed() and " + defaultBranch;
            hgCommandToObservable(innerSubscriber, ['log', '-r', revQuery], spawnOpts);
        }).subscribe(function (data) {
            hasFoundRevisions = !!(data || '').trim().length;
        }, function (err) {
            subscriber.error(err);
        }, function () {
            subscriber.next(hasFoundRevisions);
            subscriber.complete();
        });
    });
}
exports.checkIfBranchNeedsOtherBranchMergingIn = checkIfBranchNeedsOtherBranchMergingIn;
function mergeBranchIn(projPath, branchName) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next("Merging in " + branchName + "...");
        hgCommandToObservable(subscriber, ['merge', branchName], spawnOpts);
    });
}
exports.mergeBranchIn = mergeBranchIn;
function commit(projPath, message) {
    return Observable_1.Observable.create(function (subscriber) {
        var spawnOpts = { cwd: path.resolve(projPath) };
        subscriber.next("Committing...");
        hgCommandToObservable(subscriber, ['com', '-m', message], spawnOpts);
    });
}
exports.commit = commit;
function hgCommandToObservable(subscriber, hgArgs, spawnOpts) {
    var proc = child_process.spawn("hg", hgArgs, spawnOpts);
    proc.setMaxListeners(0);
    proc.stdout.on('data', function (buffer) { return subscriber.next(buffer.toString()); });
    proc.stderr.on('data', function (buffer) { return subscriber.error(new Error(buffer.toString())); });
    proc.on('error', function (err) {
        subscriber.error(err);
        subscriber.complete();
    });
    proc.on('exit', function (_) {
        subscriber.complete();
    });
    process.on('exit', function () { return proc.kill(); });
}
